@@ -177,9 +177,16 @@ PORT_WEB_UI_SSL = PORT_WEB_UI + 1
 # IP of the docker bridge used to enable access between containers
 DOCKER_BRIDGE_IP = os.environ.get('DOCKER_BRIDGE_IP', '').strip()
 
+# whether to enable API-based updates of configuration variables at runtime
+ENABLE_CONFIG_UPDATES = is_env_true('ENABLE_CONFIG_UPDATES')
+
 # CORS settings
+DISABLE_CORS_CHECKS = is_env_true('DISABLE_CORS_CHECKS')
+DISABLE_CUSTOM_CORS_S3 = is_env_true('DISABLE_CUSTOM_CORS_S3')
+DISABLE_CUSTOM_CORS_APIGATEWAY = is_env_true('DISABLE_CUSTOM_CORS_APIGATEWAY')
 EXTRA_CORS_ALLOWED_HEADERS = os.environ.get('EXTRA_CORS_ALLOWED_HEADERS', '').strip()
 EXTRA_CORS_EXPOSE_HEADERS = os.environ.get('EXTRA_CORS_EXPOSE_HEADERS', '').strip()
+EXTRA_CORS_ALLOWED_ORIGINS = os.environ.get('CORS_ALLOWED_ORIGINS', '').strip()
 
 # whether to disable publishing events to the API
 DISABLE_EVENTS = is_env_true('DISABLE_EVENTS')
