@@ -2,8 +2,6 @@ import re
 import json
 import logging
 from urllib.parse import parse_qs, urlparse
-
-from jsonpatch import apply_patch
 from moto.core.utils import camelcase_to_underscores
 from moto.apigateway import models as apigateway_models
 from moto.apigateway.models import Resource, Integration
@@ -14,31 +12,16 @@ from localstack import config
 from localstack.constants import TEST_AWS_ACCOUNT_ID
 from localstack.utils.common import short_uid, to_str, DelSafeDict
 from localstack.services.infra import start_moto_server
+from localstack.services.apigateway.helpers import apply_json_patch_safe
 
 LOG = logging.getLogger(__name__)
 
+TRUE_STRINGS = ['true', 'True']
+
 # additional REST API attributes
 REST_API_ATTRIBUTES = ['disableExecuteApiEndpoint', 'apiKeySource', 'minimumCompressionSize']
 
 
-def apply_json_patch_safe(subject, patch_operations, in_place=True):
-    results = []
-    for operation in patch_operations:
-        try:
-            if not operation.get('value'):
-                LOG.info('Missing "value" in JSONPatch operation for %s: %s' % (subject, operation))
-                continue
-            result = apply_patch(subject, [operation], in_place=in_place)
-            results.append(result)
-        except Exception as e:
-            if operation['op'] == 'replace' and 'non-existent object' in str(e):
-                # fall back to an ADD operation if the REPLACE fails
-                operation['op'] = 'add'
-                return apply_patch(subject, [operation], in_place=in_place)
-            raise
-    return results
-
-
 def apply_patches():
     def apigateway_models_Stage_init(self, cacheClusterEnabled=False, cacheClusterSize=None, **kwargs):
         apigateway_models_Stage_init_orig(self, cacheClusterEnabled=cacheClusterEnabled,
@@ -54,27 +37,12 @@ def apply_patches():
         resource = self.get_resource(function_id, resource_id)
         method = resource.get_method(method_type)
         if not method:
-            raise NoIntegrationDefined()
-
+            return
         return resource.resource_methods.pop(method_type)
 
-    def apigateway_models_resource_get_method(self, method_type):
-        method = self.resource_methods.get(method_type)
-        if not method:
-            raise NoIntegrationDefined()
-
-        return method
-
-    def apigateway_models_resource_get_integration(self, method_type):
-        resource_method = self.resource_methods.get(method_type, {})
-        if 'methodIntegration' not in resource_method:
-            raise NoIntegrationDefined()
-
-        return resource_method['methodIntegration']
-
     def apigateway_models_resource_delete_integration(self, method_type):
         if method_type in self.resource_methods:
-            return self.resource_methods[method_type].pop('methodIntegration')
+            return self.resource_methods[method_type].pop('methodIntegration', {})
 
         return {}
 
@@ -87,7 +55,7 @@ def apply_patches():
 
         self['passthroughBehavior'] = pass_through_behavior
         self['cacheKeyParameters'] = cache_key_parameters
-        self['cacheNamespace'] = short_uid()
+        self['cacheNamespace'] = self.get('cacheNamespace') or short_uid()
 
         # httpMethod not present in response if integration_type is None, verified against AWS
         if integration_type == 'MOCK':
@@ -95,20 +63,36 @@ def apply_patches():
         if request_templates:
             self['requestTemplates'] = request_templates
 
-    def apigateway_models_backend_put_rest_api(self, function_id, body):
+    def apigateway_models_backend_put_rest_api(self, function_id, body, query_params):
         rest_api = self.get_rest_api(function_id)
         # Remove default root, then add paths from API spec
         rest_api.resources = {}
-        for path in body.get('paths', {}):
+
+        def get_or_create_path(path):
+            parts = path.rstrip('/').replace('//', '/').split('/')
+            parent_id = ''
+            if len(parts) > 1:
+                parent_path = '/'.join(parts[:-1])
+                parent = get_or_create_path(parent_path)
+                parent_id = parent.id
+            existing = [r for r in rest_api.resources.values() if
+                r.path_part == (parts[-1] or '/') and
+                (r.parent_id or '') == (parent_id or '')]
+            if existing:
+                return existing[0]
+            return add_path(path, parts, parent_id=parent_id)
+
+        def add_path(path, parts, parent_id=''):
             child_id = create_id()
+            path = path or '/'
             child = Resource(
                 id=child_id,
                 region_name=rest_api.region_name,
                 api_id=rest_api.id,
-                path_part=path,
-                parent_id='',
+                path_part=parts[-1] or '/',
+                parent_id=parent_id
             )
-            for m, payload in body['paths'][path].items():
+            for m, payload in body['paths'].get(path, {}).items():
                 m = m.upper()
                 payload = payload['x-amazon-apigateway-integration']
 
@@ -131,11 +115,25 @@ def apply_patches():
                 child.resource_methods[m]['methodIntegration'] = integration
 
             rest_api.resources[child_id] = child
+            return child
+
+        basepath_mode = (query_params.get('basepath') or ['prepend'])[0]
+        base_path = (body.get('basePath') or '') if basepath_mode == 'prepend' else ''
+        for path in body.get('paths', {}):
+            get_or_create_path(base_path + path)
+
+        policy = body.get('x-amazon-apigateway-policy')
+        if policy:
+            policy = json.dumps(policy) if isinstance(policy, dict) else str(policy)
+            rest_api.policy = policy
+        minimum_compression_size = body.get('x-amazon-apigateway-minimum-compression-size')
+        if minimum_compression_size is not None:
+            rest_api.minimum_compression_size = int(minimum_compression_size)
 
         return rest_api
 
-    # Implement import rest_api
-    # https://github.com/localstack/localstack/issues/2763
+    # import rest_api
+
     def apigateway_response_restapis_individual(self, request, full_url, headers):
         if request.method in ['GET', 'DELETE']:
             return apigateway_response_restapis_individual_orig(self, request, full_url, headers)
@@ -144,30 +142,37 @@ def apply_patches():
         function_id = self.path.replace('/restapis/', '', 1).split('/')[0]
 
         if self.method == 'PATCH':
-            not_supported_attributes = ['/id', '/region_name', '/create_date']
+            not_supported_attributes = ['/id', '/region_name', '/createdDate']
 
             rest_api = self.backend.apis.get(function_id)
             if not rest_api:
                 msg = 'Invalid API identifier specified %s:%s' % (TEST_AWS_ACCOUNT_ID, function_id)
-                return (404, {}, msg)
+                return 404, {}, msg
 
             patch_operations = self._get_param('patchOperations')
+            model_attributes = list(rest_api.__dict__.keys())
             for operation in patch_operations:
-                if operation['path'].strip('/') in REST_API_ATTRIBUTES:
-                    operation['path'] = camelcase_to_underscores(operation['path'])
                 if operation['path'] in not_supported_attributes:
                     msg = 'Invalid patch path %s' % (operation['path'])
-                    return (400, {}, msg)
+                    return 400, {}, msg
+                path_stripped = operation['path'].strip('/')
+                path_underscores = camelcase_to_underscores(path_stripped)
+                if path_stripped not in model_attributes and path_underscores in model_attributes:
+                    operation['path'] = operation['path'].replace(path_stripped, path_underscores)
 
             rest_api.__dict__ = DelSafeDict(rest_api.__dict__)
             apply_json_patch_safe(rest_api.__dict__, patch_operations, in_place=True)
 
+            # fix data types after patches have been applied
+            # if rest_api.minimum_compression_size:
+            rest_api.minimum_compression_size = int(rest_api.minimum_compression_size or -1)
+
             return 200, {}, json.dumps(self.backend.get_rest_api(function_id).to_dict())
 
         # handle import rest_api via swagger file
         if self.method == 'PUT':
             body = json.loads(to_str(self.body))
-            rest_api = self.backend.put_rest_api(function_id, body)
+            rest_api = self.backend.put_rest_api(function_id, body, self.querystring)
             return 200, {}, json.dumps(rest_api.to_dict())
 
         return 400, {}, ''
@@ -175,6 +180,16 @@ def apply_patches():
     def apigateway_response_resource_methods(self, request, *args, **kwargs):
         result = apigateway_response_resource_methods_orig(self, request, *args, **kwargs)
 
+        if self.method == 'PUT' and self._get_param('requestParameters'):
+            request_parameters = self._get_param('requestParameters')
+            url_path_parts = self.path.split('/')
+            function_id = url_path_parts[2]
+            resource_id = url_path_parts[4]
+            method_type = url_path_parts[6]
+            resource = self.backend.get_resource(function_id, resource_id)
+            resource.resource_methods[method_type]['requestParameters'] = request_parameters
+            method = resource.resource_methods[method_type]
+            result = 200, {}, json.dumps(method)
         if len(result) != 3:
             return result
         authorization_type = self._get_param('authorizationType')
@@ -189,24 +204,39 @@ def apply_patches():
 
     def apigateway_response_integrations(self, request, *args, **kwargs):
         result = apigateway_response_integrations_orig(self, request, *args, **kwargs)
-        timeout_milliseconds = self._get_param('timeoutInMillis')
-        request_parameters = self._get_param('requestParameters') or {}
-        cache_key_parameters = self._get_param('cacheKeyParameters') or []
-        content_handling = self._get_param('contentHandling')
+
+        if self.method not in ['PUT', 'PATCH']:
+            return result
+
+        url_path_parts = self.path.split('/')
+        function_id = url_path_parts[2]
+        resource_id = url_path_parts[4]
+        method_type = url_path_parts[6]
+
+        integration = self.backend.get_integration(function_id, resource_id, method_type)
+        if not integration:
+            return result
 
         if self.method == 'PUT':
-            url_path_parts = self.path.split('/')
-            function_id = url_path_parts[2]
-            resource_id = url_path_parts[4]
-            method_type = url_path_parts[6]
+            timeout_milliseconds = self._get_param('timeoutInMillis')
+            request_parameters = self._get_param('requestParameters') or {}
+            cache_key_parameters = self._get_param('cacheKeyParameters') or []
+            content_handling = self._get_param('contentHandling')
+            integration['timeoutInMillis'] = timeout_milliseconds
+            integration['requestParameters'] = request_parameters
+            integration['cacheKeyParameters'] = cache_key_parameters
+            integration['contentHandling'] = content_handling
+            return 200, {}, json.dumps(integration)
 
-            integration_response = self.backend.get_integration(function_id, resource_id, method_type)
-
-            integration_response['timeoutInMillis'] = timeout_milliseconds
-            integration_response['requestParameters'] = request_parameters
-            integration_response['cacheKeyParameters'] = cache_key_parameters
-            integration_response['contentHandling'] = content_handling
-            return 200, {}, json.dumps(integration_response)
+        if self.method == 'PATCH':
+            patch_operations = self._get_param('patchOperations')
+            apply_json_patch_safe(integration, patch_operations, in_place=True)
+            # fix data types
+            if integration.get('timeoutInMillis'):
+                integration['timeoutInMillis'] = int(integration.get('timeoutInMillis'))
+            skip_verification = (integration.get('tlsConfig') or {}).get('insecureSkipVerification')
+            if skip_verification:
+                integration['tlsConfig']['insecureSkipVerification'] = str(skip_verification) in TRUE_STRINGS
 
         return result
 
@@ -249,6 +279,112 @@ def apply_patches():
 
         return result
 
+    def backend_update_deployment(self, function_id, deployment_id, patch_operations):
+        rest_api = self.get_rest_api(function_id)
+        deployment = rest_api.get_deployment(deployment_id)
+        deployment = deployment or {}
+        apply_json_patch_safe(deployment, patch_operations, in_place=True)
+        return deployment
+
+    # define json-patch operations for backend models
+
+    def backend_model_apply_operations(self, patch_operations):
+        apply_json_patch_safe(self, patch_operations, in_place=True)
+        return self
+
+    model_classes = [apigateway_models.Authorizer, apigateway_models.Stage,
+        apigateway_models.Method, apigateway_models.MethodResponse]
+    for model_class in model_classes:
+        model_class.apply_operations = backend_model_apply_operations
+
+    # fix data types for some json-patch operation values
+
+    def method_apply_operations(self, patch_operations):
+        result = method_apply_operations_orig(self, patch_operations)
+        params = self.get('requestParameters') or {}
+        bool_params_prefixes = ['method.request.querystring', 'method.request.header']
+        list_params = ['authorizationScopes']
+        for param, value in params.items():
+            for param_prefix in bool_params_prefixes:
+                if param.startswith(param_prefix) and not isinstance(value, bool):
+                    params[param] = str(value) in TRUE_STRINGS
+        for list_param in list_params:
+            value = self.get(list_param)
+            if value and not isinstance(value, list):
+                self[list_param] = [value]
+        return result
+
+    method_apply_operations_orig = apigateway_models.Method.apply_operations
+    apigateway_models.Method.apply_operations = method_apply_operations
+
+    def method_response_apply_operations(self, patch_operations):
+        result = method_response_apply_operations_orig(self, patch_operations)
+        params = self.get('responseParameters') or {}
+        bool_params_prefixes = ['method.response.querystring', 'method.response.header']
+        for param, value in params.items():
+            for param_prefix in bool_params_prefixes:
+                if param.startswith(param_prefix) and not isinstance(value, bool):
+                    params[param] = str(value) in ['true', 'True']
+        return result
+
+    method_response_apply_operations_orig = apigateway_models.MethodResponse.apply_operations
+    apigateway_models.MethodResponse.apply_operations = method_response_apply_operations
+
+    def stage_apply_operations(self, patch_operations):
+        result = stage_apply_operations_orig(self, patch_operations)
+        key_mappings = {
+            'metrics/enabled': ('metricsEnabled', bool),
+            'logging/loglevel': ('loggingLevel', str),
+            'logging/dataTrace': ('dataTraceEnabled', bool),
+            'throttling/burstLimit': ('throttlingBurstLimit', int),
+            'throttling/rateLimit': ('throttlingRateLimit', float),
+            'caching/enabled': ('cachingEnabled', bool),
+            'caching/ttlInSeconds': ('cacheTtlInSeconds', int),
+            'caching/dataEncrypted': ('cacheDataEncrypted', bool),
+            'caching/requireAuthorizationForCacheControl': ('requireAuthorizationForCacheControl', bool),
+            'caching/unauthorizedCacheControlHeaderStrategy': ('unauthorizedCacheControlHeaderStrategy', str)
+        }
+
+        def cast_value(value, value_type):
+            if value is None:
+                return value
+            if value_type == bool:
+                return str(value) in ['true', 'True']
+            return value_type(value)
+
+        method_settings = self['methodSettings'] = self.get('methodSettings') or {}
+        for operation in patch_operations:
+            path = operation['path']
+            parts = path.strip('/').split('/')
+            if len(parts) >= 4:
+                if operation['op'] not in ['add', 'replace']:
+                    continue
+                key1 = '/'.join(parts[:-2])
+                key2 = '/%s' % key1
+                setting_key = '%s/%s' % (parts[-2], parts[-1])
+                setting_name, setting_type = key_mappings.get(setting_key)
+                # keys = [key1, key2]  # TODO remove?
+                keys = [key2]
+                for key in keys:
+                    setting = method_settings[key] = method_settings.get(key) or {}
+                    value = operation.get('value')
+                    value = cast_value(value, setting_type)
+                    setting[setting_name] = value
+            if operation['op'] == 'remove':
+                method_settings.pop(path, None)
+        return result
+
+    stage_apply_operations_orig = apigateway_models.Stage.apply_operations
+    apigateway_models.Stage.apply_operations = stage_apply_operations
+
+    # patch integration error responses
+
+    def apigateway_models_resource_get_integration(self, method_type):
+        resource_method = self.resource_methods.get(method_type, {})
+        if 'methodIntegration' not in resource_method:
+            raise NoIntegrationDefined()
+        return resource_method['methodIntegration']
+
     if not hasattr(apigateway_models.APIGatewayBackend, 'put_rest_api'):
         apigateway_response_restapis_individual_orig = APIGatewayResponse.restapis_individual
         APIGatewayResponse.restapis_individual = apigateway_response_restapis_individual
@@ -257,14 +393,18 @@ def apply_patches():
     if not hasattr(apigateway_models.APIGatewayBackend, 'delete_method'):
         apigateway_models.APIGatewayBackend.delete_method = apigateway_models_backend_delete_method
 
+    if not hasattr(apigateway_models.APIGatewayBackend, 'update_deployment'):
+        apigateway_models.APIGatewayBackend.update_deployment = backend_update_deployment
+
     apigateway_models_RestAPI_to_dict_orig = apigateway_models.RestAPI.to_dict
 
     def apigateway_models_RestAPI_to_dict(self):
         resp = apigateway_models_RestAPI_to_dict_orig(self)
         resp['policy'] = None
         if self.policy:
-            # Currently still not found any document about apigateway policy escaped format, just a workaround
-            resp['policy'] = json.dumps(json.dumps(json.loads(self.policy)))[1:-1]
+            # Strip whitespaces for TF compatibility (not entirely sure why we need double-dumps,
+            # but otherwise: "error normalizing policy JSON: invalid character 'V' after top-level value")
+            resp['policy'] = json.dumps(json.dumps(json.loads(self.policy), separators=(',', ':')))[1:-1]
         for attr in REST_API_ATTRIBUTES:
             if attr not in resp:
                 resp[attr] = getattr(self, camelcase_to_underscores(attr), None)
@@ -287,15 +427,27 @@ def apply_patches():
 
         function_id = json.loads(rest_api)['id']
         body = json.loads(request.data.decode('utf-8'))
-        self.backend.put_rest_api(function_id, body)
+        self.backend.put_rest_api(function_id, body, parsed_qs)
 
         return 200, {}, rest_api
 
-    apigateway_models.Resource.get_method = apigateway_models_resource_get_method
+    def individual_deployment(self, request, full_url, headers, *args, **kwargs):
+        result = individual_deployment_orig(self, request, full_url, headers, *args, **kwargs)
+        if self.method == 'PATCH' and len(result) >= 3 and result[2] in ['null', None, str(None)]:
+            url_path_parts = self.path.split('/')
+            function_id = url_path_parts[2]
+            deployment_id = url_path_parts[4]
+            patch_operations = self._get_param('patchOperations')
+            deployment = self.backend.update_deployment(function_id, deployment_id, patch_operations)
+            return 200, {}, json.dumps(deployment)
+        return result
+
     apigateway_models.Resource.get_integration = apigateway_models_resource_get_integration
     apigateway_models.Resource.delete_integration = apigateway_models_resource_delete_integration
     apigateway_response_resource_methods_orig = APIGatewayResponse.resource_methods
     APIGatewayResponse.resource_methods = apigateway_response_resource_methods
+    individual_deployment_orig = APIGatewayResponse.individual_deployment
+    APIGatewayResponse.individual_deployment = individual_deployment
     apigateway_response_integrations_orig = APIGatewayResponse.integrations
     APIGatewayResponse.integrations = apigateway_response_integrations
     apigateway_response_integration_responses_orig = APIGatewayResponse.integration_responses
