@@ -1,23 +1,31 @@
 import re
 import json
+import logging
 from jsonpatch import apply_patch
+from jsonpointer import JsonPointerException
 from requests.models import Response
 from six.moves.urllib import parse as urlparse
 from localstack import config
 from localstack.utils import common
-from localstack.services.generic_proxy import RegionBackend
 from localstack.constants import TEST_AWS_ACCOUNT_ID, APPLICATION_JSON, PATH_USER_REQUEST
 from localstack.utils.aws import aws_stack
-from localstack.utils.aws.aws_responses import requests_response
+from localstack.services.generic_proxy import RegionBackend
+from localstack.utils.aws.aws_responses import requests_response, requests_error_response_json
+
+LOG = logging.getLogger(__name__)
 
 # regex path patterns
 PATH_REGEX_MAIN = r'^/restapis/([A-Za-z0-9_\-]+)/[a-z]+(\?.*)?'
 PATH_REGEX_SUB = r'^/restapis/([A-Za-z0-9_\-]+)/[a-z]+/([A-Za-z0-9_\-]+)/.*'
 PATH_REGEX_SUB = r'^/restapis/([A-Za-z0-9_\-]+)/[a-z]+/([A-Za-z0-9_\-]+)/.*'
 
-PATH_REGEX_AUTHORIZER = r'^/restapis/[A-Za-z0-9_\-]+/authorizers/(.*)'
-PATH_REGEX_VALIDATOR = r'^/restapis/[A-Za-z0-9_\-]+/requestvalidators/?(.*)'
-PATH_REGEX_PATH_MAPPING = r'/domainnames/([^/]+)/basepathmappings/?(.*)'
+# path regex patterns
+PATH_REGEX_AUTHORIZERS = r'^/restapis/([A-Za-z0-9_\-]+)/authorizers/?([^?/]+)?(\?.*)?'
+PATH_REGEX_VALIDATORS = r'^/restapis/([A-Za-z0-9_\-]+)/requestvalidators/?([^?/]+)?(\?.*)?'
+PATH_REGEX_RESPONSES = r'^/restapis/([A-Za-z0-9_\-]+)/gatewayresponses(/[A-Za-z0-9_\-]+)?(\?.*)?'
+PATH_REGEX_PATH_MAPPINGS = r'/domainnames/([^/]+)/basepathmappings/?(.*)'
+PATH_REGEX_CLIENT_CERTS = r'/clientcertificates/?([^/]+)?$'
+PATH_REGEX_VPC_LINKS = r'/vpclinks/([^/]+)?(.*)'
 
 # template for SQS inbound data
 APIGATEWAY_SQS_DATA_INBOUND_TEMPLATE = "Action=SendMessage&MessageBody=$util.base64Encode($input.json('$'))"
@@ -43,17 +51,21 @@ class APIGatewayRegion(RegionBackend):
         }
         # maps (domain_name) -> [path_mappings]
         self.base_path_mappings = {}
+        # maps ID to VPC link details
+        self.vpc_links = {}
+        # maps cert ID to client certificate details
+        self.client_certificates = {}
 
 
 def make_json_response(message):
     return requests_response(json.dumps(message), headers={'Content-Type': APPLICATION_JSON})
 
 
-def make_error_response(message, code=400):
-    response = Response()
-    response.status_code = code
-    response._content = json.dumps({'message': message})
-    return response
+def make_error_response(message, code=400, error_type=None):
+    if code == 404 and not error_type:
+        error_type = 'NotFoundException'
+    error_type = error_type or 'InvalidRequest'
+    return requests_error_response_json(message, code=code, error_type=error_type)
 
 
 def make_accepted_response():
@@ -80,7 +92,7 @@ def get_account():
 
 def update_account(data):
     region_details = APIGatewayRegion.get()
-    apply_patch(region_details.account, data['patchOperations'], in_place=True)
+    apply_json_patch_safe(region_details.account, data['patchOperations'], in_place=True)
     return to_account_response_json(region_details.account)
 
 
@@ -89,7 +101,7 @@ def handle_accounts(method, path, data, headers):
         return get_account()
     if method == 'PATCH':
         return update_account(data)
-    return make_error_response('Not implemented for API Gateway accounts: %s' % method, 404)
+    return make_error_response('Not implemented for API Gateway accounts: %s' % method, code=404)
 
 
 # -----------------
@@ -97,8 +109,8 @@ def handle_accounts(method, path, data, headers):
 # -----------------
 
 def get_authorizer_id_from_path(path):
-    match = re.match(PATH_REGEX_AUTHORIZER, path)
-    return match.group(1) if match else None
+    match = re.match(PATH_REGEX_AUTHORIZERS, path)
+    return match.group(2) if match else None
 
 
 def _find_authorizer(api_id, authorizer_id):
@@ -109,10 +121,14 @@ def _find_authorizer(api_id, authorizer_id):
 
 
 def normalize_authorizer(data):
-    result = common.clone(data)
-    # terraform sends this as a string in patch, so convert to int
-    result['authorizerResultTtlInSeconds'] = int(result.get('authorizerResultTtlInSeconds') or 300)
-    return result
+    is_list = isinstance(data, list)
+    entries = data if is_list else [data]
+    for i in range(len(entries)):
+        entry = common.clone(entries[i])
+        # terraform sends this as a string in patch, so convert to int
+        entry['authorizerResultTtlInSeconds'] = int(entry.get('authorizerResultTtlInSeconds', 300))
+        entries[i] = entry
+    return entries if is_list else entries[0]
 
 
 def get_authorizers(path):
@@ -127,7 +143,8 @@ def get_authorizers(path):
     if authorizer_id:
         authorizer = _find_authorizer(api_id, authorizer_id)
         if authorizer is None:
-            return make_error_response('Not found: %s' % authorizer_id, 404)
+            return make_error_response('Authorizer not found: %s' % authorizer_id,
+                code=404, error_type='NotFoundException')
         return to_authorizer_response_json(api_id, authorizer)
 
     result = [to_authorizer_response_json(api_id, a) for a in auth_list]
@@ -139,7 +156,7 @@ def add_authorizer(path, data):
     region_details = APIGatewayRegion.get()
 
     api_id = get_api_id_from_path(path)
-    authorizer_id = common.short_uid()
+    authorizer_id = common.short_uid()[:6]  # length 6 to make TF tests pass
     result = common.clone(data)
 
     result['id'] = authorizer_id
@@ -159,9 +176,9 @@ def update_authorizer(path, data):
 
     authorizer = _find_authorizer(api_id, authorizer_id)
     if authorizer is None:
-        return make_error_response('Not found: %s' % api_id, 404)
+        return make_error_response('Authorizer not found for API: %s' % api_id, code=404)
 
-    result = apply_patch(authorizer, data['patchOperations'])
+    result = apply_json_patch_safe(authorizer, data['patchOperations'])
     result = normalize_authorizer(result)
 
     auth_list = region_details.authorizers[api_id]
@@ -190,13 +207,13 @@ def delete_authorizer(path):
 def handle_authorizers(method, path, data, headers):
     if method == 'GET':
         return get_authorizers(path)
-    elif method == 'POST':
+    if method == 'POST':
         return add_authorizer(path, data)
-    elif method == 'PATCH':
+    if method == 'PATCH':
         return update_authorizer(path, data)
-    elif method == 'DELETE':
+    if method == 'DELETE':
         return delete_authorizer(path)
-    return make_error_response('Not implemented for API Gateway authorizers: %s' % method, 404)
+    return make_error_response('Not implemented for API Gateway authorizers: %s' % method, code=404)
 
 
 # -----------------------
@@ -204,12 +221,12 @@ def handle_authorizers(method, path, data, headers):
 # -----------------------
 
 def get_domain_from_path(path):
-    matched = re.match(PATH_REGEX_PATH_MAPPING, path)
+    matched = re.match(PATH_REGEX_PATH_MAPPINGS, path)
     return matched.group(1) if matched else None
 
 
 def get_base_path_from_path(path):
-    return re.match(PATH_REGEX_PATH_MAPPING, path).group(2)
+    return re.match(PATH_REGEX_PATH_MAPPINGS, path).group(2)
 
 
 def get_base_path_mapping(path):
@@ -224,7 +241,8 @@ def get_base_path_mapping(path):
     if base_path:
         mapping = ([m for m in mappings_list if m['basePath'] == base_path] or [None])[0]
         if mapping is None:
-            return make_error_response('Not found: %s' % base_path, 404)
+            return make_error_response('Base path mapping not found: %s' % base_path, code=404,
+                error_type='NotFoundException')
         return to_base_mapping_response_json(domain_name, base_path, mapping)
 
     result = [to_base_mapping_response_json(domain_name, m['basePath'], m) for m in mappings_list]
@@ -258,9 +276,14 @@ def update_base_path_mapping(path, data):
     mapping = ([m for m in mappings_list if m['basePath'] == base_path] or [None])[0]
     if mapping is None:
         return make_error_response('Not found: mapping for domain name %s, base path %s' %
-            (domain_name, base_path), 404)
+            (domain_name, base_path), code=404)
 
-    result = apply_patch(mapping, data['patchOperations'])
+    operations = data['patchOperations']
+    operations = operations if isinstance(operations, list) else [operations]
+    for operation in operations:
+        if operation['path'] == '/restapiId':
+            operation['path'] = '/restApiId'
+    result = apply_json_patch_safe(mapping, operations)
 
     for i in range(len(mappings_list)):
         if mappings_list[i]['basePath'] == base_path:
@@ -281,19 +304,142 @@ def delete_base_path_mapping(path):
             del mappings_list[i]
             return make_accepted_response()
 
-    return make_error_response('Base path mapping %s for domain %s not found' % (base_path, domain_name), 404)
+    return make_error_response('Base path mapping %s for domain %s not found' % (base_path, domain_name), code=404)
 
 
-def hande_base_path_mappings(method, path, data, headers):
+def handle_base_path_mappings(method, path, data, headers):
     if method == 'GET':
         return get_base_path_mapping(path)
-    elif method == 'POST':
+    if method == 'POST':
         return add_base_path_mapping(path, data)
     if method == 'PATCH':
         return update_base_path_mapping(path, data)
-    elif method == 'DELETE':
+    if method == 'DELETE':
         return delete_base_path_mapping(path)
-    return make_error_response('Not implemented for API Gateway base path mappings: %s' % method, 404)
+    return make_error_response('Not implemented for API Gateway base path mappings: %s' % method, code=404)
+
+
+# ------------------------
+# CLIENT CERTIFICATE APIs
+# ------------------------
+
+def get_cert_id_from_path(path):
+    matched = re.match(PATH_REGEX_CLIENT_CERTS, path)
+    return matched.group(1) if matched else None
+
+
+def get_client_certificate(path):
+    region_details = APIGatewayRegion.get()
+    cert_id = get_cert_id_from_path(path)
+    result = region_details.client_certificates.get(cert_id)
+    if result is None:
+        return make_error_response('Client certificate ID "%s" not found' % cert_id, code=404)
+    return result
+
+
+def add_client_certificate(path, data):
+    region_details = APIGatewayRegion.get()
+    result = common.clone(data)
+    result['clientCertificateId'] = cert_id = common.short_uid()
+    result['createdDate'] = common.now_utc()
+    result['expirationDate'] = result['createdDate'] + 60 * 60 * 24 * 30  # assume 30 days validity
+    result['pemEncodedCertificate'] = 'testcert-123'  # TODO return proper certificate!
+    region_details.client_certificates[cert_id] = result
+    return make_json_response(to_client_cert_response_json(result))
+
+
+def update_client_certificate(path, data):
+    region_details = APIGatewayRegion.get()
+    entity_id = get_cert_id_from_path(path)
+    entity = region_details.client_certificates.get(entity_id)
+    if entity is None:
+        return make_error_response('Client certificate ID "%s" not found' % entity_id, code=404)
+    result = apply_json_patch_safe(entity, data['patchOperations'])
+    return make_json_response(to_client_cert_response_json(result))
+
+
+def delete_client_certificate(path):
+    region_details = APIGatewayRegion.get()
+    entity_id = get_cert_id_from_path(path)
+    entity = region_details.client_certificates.pop(entity_id, None)
+    if entity is None:
+        return make_error_response('VPC link ID "%s" not found for deletion' % entity_id, code=404)
+    return make_accepted_response()
+
+
+def handle_client_certificates(method, path, data, headers):
+    if method == 'GET':
+        return get_client_certificate(path)
+    if method == 'POST':
+        return add_client_certificate(path, data)
+    if method == 'PATCH':
+        return update_client_certificate(path, data)
+    if method == 'DELETE':
+        return delete_client_certificate(path)
+    return make_error_response('Not implemented for API Gateway base path mappings: %s' % method, code=404)
+
+
+# --------------
+# VCP LINK APIs
+# --------------
+
+def get_vpc_links(path):
+    region_details = APIGatewayRegion.get()
+    vpc_link_id = get_vpc_link_id_from_path(path)
+    if vpc_link_id:
+        vpc_link = region_details.vpc_links.get(vpc_link_id)
+        if vpc_link is None:
+            return make_error_response('VPC link ID "%s" not found' % vpc_link_id, code=404)
+        return make_json_response(to_vpc_link_response_json(vpc_link))
+    result = region_details.vpc_links.values()
+    result = [to_vpc_link_response_json(r) for r in result]
+    result = {'items': result}
+    return result
+
+
+def add_vpc_link(path, data):
+    region_details = APIGatewayRegion.get()
+    result = common.clone(data)
+    result['id'] = common.short_uid()
+    result['status'] = 'AVAILABLE'
+    region_details.vpc_links[result['id']] = result
+    return make_json_response(to_vpc_link_response_json(result))
+
+
+def update_vpc_link(path, data):
+    region_details = APIGatewayRegion.get()
+    vpc_link_id = get_vpc_link_id_from_path(path)
+    vpc_link = region_details.vpc_links.get(vpc_link_id)
+    if vpc_link is None:
+        return make_error_response('VPC link ID "%s" not found' % vpc_link_id, code=404)
+    result = apply_json_patch_safe(vpc_link, data['patchOperations'])
+    return make_json_response(to_vpc_link_response_json(result))
+
+
+def delete_vpc_link(path):
+    region_details = APIGatewayRegion.get()
+    vpc_link_id = get_vpc_link_id_from_path(path)
+    vpc_link = region_details.vpc_links.pop(vpc_link_id, None)
+    if vpc_link is None:
+        return make_error_response('VPC link ID "%s" not found for deletion' % vpc_link_id, code=404)
+    return make_accepted_response()
+
+
+def get_vpc_link_id_from_path(path):
+    match = re.match(PATH_REGEX_VPC_LINKS, path)
+    return match.group(1) if match else None
+
+
+def handle_vpc_links(method, path, data, headers):
+    if method == 'GET':
+        return get_vpc_links(path)
+    if method == 'POST':
+        return add_vpc_link(path, data)
+    if method == 'PATCH':
+        return update_vpc_link(path, data)
+    if method == 'DELETE':
+        return delete_vpc_link(path)
+    return make_error_response('Not implemented for API Gateway VPC links: %s' % method, code=404)
 
 
 # ----------------
@@ -301,8 +447,8 @@ def hande_base_path_mappings(method, path, data, headers):
 # ----------------
 
 def get_validator_id_from_path(path):
-    match = re.match(PATH_REGEX_VALIDATOR, path)
-    return match.group(1) if match else None
+    match = re.match(PATH_REGEX_VALIDATORS, path)
+    return match.group(2) if match else None
 
 
 def _find_validator(api_id, validator_id):
@@ -324,7 +470,8 @@ def get_validators(path):
     if validator_id:
         validator = _find_validator(api_id, validator_id)
         if validator is None:
-            return make_error_response('Validator %s for API Gateway %s not found' % (validator_id, api_id), 404)
+            return make_error_response('Validator %s for API Gateway %s not found' %
+                (validator_id, api_id), code=404)
         return to_validator_response_json(api_id, validator)
 
     result = [to_validator_response_json(api_id, a) for a in auth_list]
@@ -336,7 +483,7 @@ def add_validator(path, data):
     region_details = APIGatewayRegion.get()
 
     api_id = get_api_id_from_path(path)
-    validator_id = common.short_uid()
+    validator_id = common.short_uid()[:6]  # length 6 (as in AWS) to make TF tests pass
     result = common.clone(data)
     result['id'] = validator_id
 
@@ -354,9 +501,9 @@ def update_validator(path, data):
 
     validator = _find_validator(api_id, validator_id)
     if validator is None:
-        return make_error_response('Validator %s for API Gateway %s not found' % (validator_id, api_id), 404)
+        return make_error_response('Validator %s for API Gateway %s not found' % (validator_id, api_id), code=404)
 
-    result = apply_patch(validator, data['patchOperations'])
+    result = apply_json_patch_safe(validator, data['patchOperations'])
 
     entry_list = region_details.validators[api_id]
     for i in range(len(entry_list)):
@@ -378,19 +525,19 @@ def delete_validator(path):
             del auth_list[i]
             return make_accepted_response()
 
-    return make_error_response('Validator %s for API Gateway %s not found' % (validator_id, api_id), 404)
+    return make_error_response('Validator %s for API Gateway %s not found' % (validator_id, api_id), code=404)
 
 
 def handle_validators(method, path, data, headers):
     if method == 'GET':
         return get_validators(path)
-    elif method == 'POST':
+    if method == 'POST':
         return add_validator(path, data)
-    elif method == 'PATCH':
+    if method == 'PATCH':
         return update_validator(path, data)
-    elif method == 'DELETE':
+    if method == 'DELETE':
         return delete_validator(path)
-    return make_error_response('Not implemented for API Gateway validators: %s' % method, 404)
+    return make_error_response('Not implemented for API Gateway validators: %s' % method, code=404)
 
 
 # ---------------
@@ -414,10 +561,23 @@ def to_account_response_json(data):
     return to_response_json('account', data, self_link='/account')
 
 
-def to_response_json(model_type, data, api_id=None, self_link=None):
+def to_vpc_link_response_json(data):
+    return to_response_json('vpclink', data)
+
+
+def to_client_cert_response_json(data):
+    return to_response_json('clientcertificate', data, id_attr='clientCertificateId')
+
+
+def to_response_json(model_type, data, api_id=None, self_link=None, id_attr=None):
+    if isinstance(data, list) and len(data) == 1:
+        data = data[0]
+    id_attr = id_attr or 'id'
     result = common.clone(data)
     if not self_link:
-        self_link = '/restapis/%s/%ss/%s' % (api_id, model_type, data['id'])
+        self_link = '/%ss/%s' % (model_type, data[id_attr])
+        if api_id:
+            self_link = '/restapis/%s/%s' % (api_id, self_link)
     if '_links' not in result:
         result['_links'] = {}
     result['_links']['self'] = {'href': self_link}
@@ -553,3 +713,40 @@ def connect_api_gateway_to_sqs(gateway_name, stage_name, queue_arn, path, region
     }]
     return aws_stack.create_api_gateway(
         name=gateway_name, resources=resources, stage_name=stage_name, region_name=region_name)
+
+
+def apply_json_patch_safe(subject, patch_operations, in_place=True, return_list=False):
+    results = []
+    for operation in patch_operations:
+        try:
+            # special case: for "replace" operations, assume "" as the default value
+            if operation['op'] == 'replace' and operation.get('value') is None:
+                operation['value'] = ''
+
+            if operation['op'] != 'remove' and operation.get('value') is None:
+                LOG.info('Missing "value" in JSONPatch operation for %s: %s' % (subject, operation))
+                continue
+
+            # special case: if "path" is an attribute name pointing to an array in "subject", and we're
+            # running an "add" operation, then we should use the standard-compliant notation "/path/-"
+            if operation['op'] == 'add' and isinstance(subject.get(operation['path'].strip('/')), list):
+                operation['path'] = '%s/-' % operation['path']
+
+            result = apply_patch(subject, [operation], in_place=in_place)
+            if not in_place:
+                subject = result
+            results.append(result)
+        except JsonPointerException:
+            pass  # path cannot be found - ignore
+        except Exception as e:
+            if 'non-existent object' in str(e):
+                if operation['op'] == 'replace':
+                    # fall back to an ADD operation if the REPLACE fails
+                    operation['op'] = 'add'
+                    return apply_patch(subject, [operation], in_place=in_place)
+                if operation['op'] == 'remove' and isinstance(subject, dict):
+                    subject.pop(operation['path'], None)
+            raise
+    if return_list:
+        return results
+    return (results or [subject])[-1]
