@@ -157,8 +157,8 @@ def log_duration(name=None):
             finally:
                 end_time = now_utc(millis=True)
                 func_name = name or f.__name__
-                duration = (end_time - start_time) * 1000
+                duration = end_time - start_time
                 if duration > 500:
-                    LOG.info('Execution of "%s" took %sms' % (func_name, duration))
+                    LOG.info('Execution of "%s" took %.2fms', func_name, duration)
         return wrapped
     return wrapper
